package sun.net.www.protocol.http;

import java.util.StringTokenizer;
import java.util.Random;
import java.io.Serializable;
import java.security.AccessController;
import sun.net.NetProperties;
import java.security.PrivilegedAction;
import java.util.Arrays;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.net.ProtocolException;
import java.security.NoSuchAlgorithmException;
import java.io.IOException;
import sun.net.www.HeaderParser;
import java.net.PasswordAuthentication;
import java.net.URL;

class DigestAuthentication extends AuthenticationInfo
{
    private static final long serialVersionUID = 100L;
    private String authMethod;
    private static final String compatPropName = "http.auth.digest.quoteParameters";
    private static final boolean delimCompatFlag;
    DigestAuthentication.Parameters params;
    private static final char[] charArray;
    private static final String[] zeroPad;
    
    public DigestAuthentication(boolean b, URL url, String s, String authMethod, PasswordAuthentication pw, DigestAuthentication.Parameters params) {
        super(b ? 'p' : 's', AuthScheme.DIGEST, url, s);
        this.authMethod = authMethod;
        this.pw = pw;
        this.params = params;
    }
    
    public DigestAuthentication(boolean b, String s, int n, String s2, String authMethod, PasswordAuthentication pw, DigestAuthentication.Parameters params) {
        super(b ? 'p' : 's', AuthScheme.DIGEST, s, n, s2);
        this.authMethod = authMethod;
        this.pw = pw;
        this.params = params;
    }
    
    @Override
    public boolean supportsPreemptiveAuthorization() {
        return true;
    }
    
    @Override
    public String getHeaderValue(URL url, String s) {
        return this.getHeaderValueImpl(url.getFile(), s);
    }
    
    String getHeaderValue(String s, String s2) {
        return this.getHeaderValueImpl(s, s2);
    }
    
    @Override
    public boolean isAuthorizationStale(String raw) {
        HeaderParser headerParser = new HeaderParser(raw);
        String value = headerParser.findValue("stale");
        if (value == null || !value.equals("true")) {
            return false;
        }
        String value2 = headerParser.findValue("nonce");
        if (value2 == null || "".equals(value2)) {
            return false;
        }
        this.params.setNonce(value2);
        return true;
    }
    
    @Override
    public boolean setHeaders(HttpURLConnection httpURLConnection, HeaderParser headerParser, String s) {
        this.params.setNonce(headerParser.findValue("nonce"));
        this.params.setOpaque(headerParser.findValue("opaque"));
        this.params.setQop(headerParser.findValue("qop"));
        String s2 = "";
        String s3;
        if (this.type == 'p' && httpURLConnection.tunnelState() == HttpURLConnection.TunnelState.SETUP) {
            s2 = HttpURLConnection.connectRequestURI(httpURLConnection.getURL());
            s3 = HttpURLConnection.HTTP_CONNECT;
        }
        else {
            try {
                s2 = httpURLConnection.getRequestURI();
            }
            catch (IOException ex) {}
            s3 = httpURLConnection.getMethod();
        }
        if (this.params.nonce == null || this.authMethod == null || this.pw == null || this.realm == null) {
            return false;
        }
        if (this.authMethod.length() >= 1) {
            this.authMethod = Character.toUpperCase(this.authMethod.charAt(0)) + this.authMethod.substring(1).toLowerCase();
        }
        String value = headerParser.findValue("algorithm");
        if (value == null || "".equals(value)) {
            value = "MD5";
        }
        this.params.setAlgorithm(value);
        if (this.params.authQop()) {
            this.params.setNewCnonce();
        }
        String headerValueImpl = this.getHeaderValueImpl(s2, s3);
        if (headerValueImpl != null) {
            httpURLConnection.setAuthenticationProperty(this.getHeaderName(), headerValueImpl);
            return true;
        }
        return false;
    }
    
    private String getHeaderValueImpl(String str, String s) {
        char[] password = this.pw.getPassword();
        int authQop = this.params.authQop();
        String opaque = this.params.getOpaque();
        String cnonce = this.params.getCnonce();
        String nonce = this.params.getNonce();
        String algorithm = this.params.getAlgorithm();
        this.params.incrementNC();
        int ncCount = this.params.getNCCount();
        String s2 = null;
        if (ncCount != -1) {
            s2 = Integer.toHexString(ncCount).toLowerCase();
            int length = s2.length();
            if (length < 8) {
                s2 = DigestAuthentication.zeroPad[length] + s2;
            }
        }
        String computeDigest;
        try {
            computeDigest = this.computeDigest(true, this.pw.getUserName(), password, this.realm, s, str, nonce, cnonce, s2);
        }
        catch (NoSuchAlgorithmException ex) {
            return null;
        }
        String string = "\"";
        if (!(authQop == 0)) {
            string = "\", nc=" + s2;
        }
        String str2;
        String str3;
        if (DigestAuthentication.delimCompatFlag) {
            str2 = ", algorithm=\"" + algorithm + "\"";
            str3 = ", qop=\"auth\"";
        }
        else {
            str2 = ", algorithm=" + algorithm;
            str3 = ", qop=auth";
        }
        String str4 = this.authMethod + " username=\"" + this.pw.getUserName() + "\", realm=\"" + this.realm + "\", nonce=\"" + nonce + string + ", uri=\"" + str + "\", response=\"" + computeDigest + "\"" + str2;
        if (opaque != null) {
            str4 = str4 + ", opaque=\"" + opaque + "\"";
        }
        if (cnonce != null) {
            str4 = str4 + ", cnonce=\"" + cnonce + "\"";
        }
        if (!(authQop == 0)) {
            str4 += str3;
            return str4;
        }
        return str4;
    }
    
    public void checkResponse(String s, String s2, URL url) throws IOException {
        this.checkResponse(s, s2, url.getFile());
    }
    
    public void checkResponse(String raw, String s, String s2) throws IOException {
        char[] password = this.pw.getPassword();
        String userName = this.pw.getUserName();
        this.params.authQop();
        this.params.getOpaque();
        String access$100 = this.params.cnonce;
        String nonce = this.params.getNonce();
        this.params.getAlgorithm();
        int ncCount = this.params.getNCCount();
        String str = null;
        if (raw == null) {
            throw new ProtocolException("No authentication information in response");
        }
        if (ncCount != -1) {
            str = Integer.toHexString(ncCount).toUpperCase();
            int length = str.length();
            if (length < 8) {
                str = DigestAuthentication.zeroPad[length] + str;
            }
        }
        try {
            String computeDigest = this.computeDigest(false, userName, password, this.realm, s, s2, nonce, access$100, str);
            HeaderParser o = new HeaderParser(raw);
            String value = o.findValue("rspauth");
            if (value == null) {
                throw new ProtocolException("No digest in response");
            }
            if (!value.equals(computeDigest)) {
                throw new ProtocolException("Response digest invalid");
            }
            String value2 = o.findValue("nextnonce");
            if (value2 != null && !"".equals(value2)) {
                this.params.setNonce(value2);
            }
        }
        catch (NoSuchAlgorithmException ex) {
            throw new ProtocolException("Unsupported algorithm in response");
        }
        return;
    }
    
    private String computeDigest(boolean b, String s, char[] array, String s2, String str, String s3, String str2, String s4, String str3) throws NoSuchAlgorithmException {
        String algorithm = this.params.getAlgorithm();
        int equalsIgnoreCase = algorithm.equalsIgnoreCase("MD5-sess");
        MessageDigest instance = MessageDigest.getInstance(equalsIgnoreCase ? "MD5" : algorithm);
        String str4;
        if (!(equalsIgnoreCase == 0)) {
            if ((str4 = this.params.getCachedHA1()) == null) {
                str4 = this.encode(this.encode(s + ":" + s2 + ":", array, instance) + ":" + str2 + ":" + s4, null, instance);
                this.params.setCachedHA1(str4);
            }
        }
        else {
            str4 = this.encode(s + ":" + s2 + ":", array, instance);
        }
        String s5;
        if (b) {
            s5 = str + ":" + s3;
        }
        else {
            s5 = ":" + s3;
        }
        String encode = this.encode(s5, null, instance);
        String s6;
        if (this.params.authQop()) {
            s6 = str4 + ":" + str2 + ":" + str3 + ":" + s4 + ":auth:" + encode;
        }
        else {
            s6 = str4 + ":" + str2 + ":" + encode;
        }
        return this.encode(s6, null, instance);
    }
    
    private String encode(String s, char[] array, MessageDigest messageDigest) {
        try {
            messageDigest.update(s.getBytes("ISO-8859-1"));
        }
        catch (UnsupportedEncodingException ex) {
            assert false;
        }
        if (array != null) {
            byte[] array2 = new byte[array.length];
            for (int i = 0; i < array.length; ++i) {
                array2[i] = (byte)array[i];
            }
            messageDigest.update(array2);
            Arrays.fill(array2, (byte)0);
        }
        byte[] digest = messageDigest.digest();
        StringBuffer o = new StringBuffer(digest.length * 2);
        for (int j = 0; j < digest.length; ++j) {
            o.append(DigestAuthentication.charArray[digest[j] >>> 4 & 0xF]);
            o.append(DigestAuthentication.charArray[digest[j] & 0xF]);
        }
        return o.toString();
    }
    
    static {
        Boolean b = (Boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                return NetProperties.getBoolean("http.auth.digest.quoteParameters");
            }
        });
        delimCompatFlag = (b != null && (boolean)b);
        charArray = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
        zeroPad = new String[] { "00000000", "0000000", "000000", "00000", "0000", "000", "00", "0" };
    }
    
    static class Parameters implements Serializable
    {
        private static final long serialVersionUID = -3584543755194526252L;
        private boolean serverQop;
        private String opaque;
        private String cnonce;
        private String nonce;
        private String algorithm;
        private int NCcount;
        private String cachedHA1;
        private boolean redoCachedHA1;
        private static final int cnonceRepeat = 5;
        private static final int cnoncelen = 40;
        private static Random random;
        int cnonce_count;
        
        Parameters() {
            this.NCcount = 0;
            this.redoCachedHA1 = true;
            this.cnonce_count = 0;
            this.serverQop = false;
            this.opaque = null;
            this.algorithm = null;
            this.cachedHA1 = null;
            this.nonce = null;
            this.setNewCnonce();
        }
        
        boolean authQop() {
            return this.serverQop;
        }
        
        synchronized void incrementNC() {
            ++this.NCcount;
        }
        
        synchronized int getNCCount() {
            return this.NCcount;
        }
        
        synchronized String getCnonce() {
            if (this.cnonce_count >= 5) {
                this.setNewCnonce();
            }
            ++this.cnonce_count;
            return this.cnonce;
        }
        
        synchronized void setNewCnonce() {
            byte[] bytes = new byte[20];
            char[] value = new char[40];
            DigestAuthentication.Parameters.random.nextBytes(bytes);
            for (int i = 0; i < 20; ++i) {
                int n = bytes[i] + 128;
                value[i * 2] = (char)(65 + n / 16);
                value[i * 2 + 1] = (char)(65 + n % 16);
            }
            this.cnonce = new String(value, 0, 40);
            this.cnonce_count = 0;
            this.redoCachedHA1 = true;
        }
        
        synchronized void setQop(String str) {
            if (str != null) {
                StringTokenizer stringTokenizer = new StringTokenizer(str, " ");
                while (stringTokenizer.hasMoreTokens()) {
                    if (!stringTokenizer.nextToken().equalsIgnoreCase("auth")) {
                        continue;
                    }
                    this.serverQop = true;
                    return;
                }
            }
            this.serverQop = false;
        }
        
        synchronized String getOpaque() {
            return this.opaque;
        }
        
        synchronized void setOpaque(String opaque) {
            this.opaque = opaque;
        }
        
        synchronized String getNonce() {
            return this.nonce;
        }
        
        synchronized void setNonce(String nonce) {
            if (!nonce.equals(this.nonce)) {
                this.nonce = nonce;
                this.NCcount = 0;
                this.redoCachedHA1 = true;
            }
        }
        
        synchronized String getCachedHA1() {
            if (this.redoCachedHA1) {
                return null;
            }
            return this.cachedHA1;
        }
        
        synchronized void setCachedHA1(String cachedHA1) {
            this.cachedHA1 = cachedHA1;
            this.redoCachedHA1 = false;
        }
        
        synchronized String getAlgorithm() {
            return this.algorithm;
        }
        
        synchronized void setAlgorithm(String algorithm) {
            this.algorithm = algorithm;
        }
        
        static {
            DigestAuthentication.Parameters.random = new Random();
        }
    }
}
